<select style="width:70px" class="valueComparison">
    <option value="lt">&lt;</option>
    <option value="lte">&le;</option>
    <option value="=">=</option>
    <option value="gte">&ge;</option>
    <option value="gt">&gt;</option>
</select>