<p id="crit0">
    {% include 'core_explore_example_app/user/query_builder/yes_no.html'%}
    {% include 'core_explore_example_app/user/query_builder/field_input.html' with tagID=0 %}
    <span id="ui0">
    </span>
    {% include 'core_explore_example_app/user/query_builder/add.html'%}
</p>