{% extends 'core_main_app/_render/admin/theme/tools/modal.html' %}

{% block modal_id %}errors-modal{% endblock %}
{% block modal_title %}Errors{% endblock %}

{% block modal_body %}
<p id="listErrors" style="color: red;"></p>
{% endblock %}
{% block modal_footer %}
<a class="btn btn-secondary pull-left" data-dismiss="modal" href="#"><i class="fas fa-times"></i> Cancel</a>
{% endblock %}
