{% extends 'core_main_app/_render/admin/theme/tools/modal.html' %}

{% block modal_id %}delete-all-queries-modal{% endblock %}
{% block modal_title %}Delete All Queries{% endblock %}

{% block modal_body %}
<p>Are you sure you want to delete all your saved queries?</p>
{% endblock %}
{% block modal_footer %}
<a class="btn btn-secondary pull-left" data-dismiss="modal" href="#"><i class="fas fa-times"></i> Cancel</a>
<button id="delete-all-queries" class="btn btn-danger"><i class="fas fa-trash"></i> Delete</button>
{% endblock %}

