#queryBuilder
{
	z-index:0;
	overflow-y: auto;
}

#queryBuilder .elementInput
{
	width: 230px;
	padding-left: 5px;
}

#queryBuilder .queryInput
{
	width:460px;
}

#queryBuilder .valueInput
{
	width:150px;
}

#queryBuilder select.valueInput
{
	width:225px;
}

#queryBuilder input{
    height: 2em;
    font-size: 0.9em;
    padding: 0;
    display: inline;
}

#queryBuilder input:hover {
    background-color: #eaeaea;
}

#queryBuilder input[readonly]
{
	cursor:pointer;
}

#queryBuilder p
{
	font-weight: bold;
	margin: 1em;
}

#subElementQueryBuilder .valueInput{
    height: 2em;
    width: 150px;
    display: inline;
    padding: 0;
}

.action-min-width {
    min-width: 250px;
}

.comparison-text {
    margin: .5em;
}
