<p id='crit{{tagID}}'>
{% include 'core_explore_example_app/user/query_builder/and_or_not.html'%}
    {% include 'core_explore_example_app/user/query_builder/field_input.html' with tagID=tagID%}
    <span id='ui{{tagID}}'>
    </span>
    {% include 'core_explore_example_app/user/query_builder/remove.html' with tagID=tagID%}
    {% include 'core_explore_example_app/user/query_builder/add.html'%}
</p>