<h1>Select Fields</h1>
<p>
Select the fields that you want to use in your query by clicking on the checkboxes. Only the checked fields will be
available in the query screen. When you are done, please click on 'Go to Query Builder'.
</p>

<button class="btn btn-primary save-fields">
    <i class="fas fa-play"></i> Go to Query Builder
</button>

<div id="xsd_form">
    {% csrf_token %}
    {{ data.xsd_form | safe }}
</div>

<div id="template_id" class="hidden">{{data.template_id}}</div>