{% load static %}

<div class="title-page">Select {{ data.object_name|title  }}</div>

Select a {{ data.object_name }} from the following table. It will automatically load the appropriate form and display it on the next page.

<div id="template_selection">
    <h3 class="text-center">Templates</h3>
    {% if data.global_objects or data.user_objects %}
        <table class="table fixed-table">
            <colgroup>
                <col class="col-width-75">
                <col class="col-width-25">
            </colgroup>
            {% include 'core_main_app/user/template/list_header.html' with object_type='Template' %}
            {% include 'core_explore_example_app/user/list/list.html' with object_list=data.global_objects owner='Global' object_type='Template' %}
            {% include 'core_explore_example_app/user/list/list.html' with object_list=data.user_objects owner=request.user object_type='Template' %}
        </table>
    {% else %}
        <p class="text-center">No template uploaded yet.</p>
    {% endif %}
</div>
