{% extends 'core_main_app/_render/admin/theme/tools/modal.html' %}

{% block modal_id %}delete-query-modal{% endblock %}
{% block modal_title %}Delete Query{% endblock %}

{% block modal_body %}
<p>Are you sure you want to delete this saved query?</p>
<div id="saved_query_id" style="display:none;"></div>
{% endblock %}
{% block modal_footer %}
<button class="btn btn-secondary pull-left" data-dismiss="modal"><i class="fas fa-times"></i> Cancel</button>
<button id="delete-query" class="btn btn-danger"><i class="fas fa-trash"></i> Delete</button>
{% endblock %}
