{% extends 'core_main_app/user/template/list_body.html' %}

{% block actions %}
<a href="{% url data.select_object_redirect template.current %}"
       class="btn btn-primary" role="button">
    <i class="fas fa-edit"></i> Select Query Fields
</a>
<a href="{% url data.build_query_redirect template.current %}"
       class="btn btn-success" role="button">
    <i class="fas fa-play"></i> Go to Query Builder
</a>
{% endblock %}
